package nq.net.assets {

    import flash.events.Event;
    import flash.events.EventDispatcher;

    import nq.net.assets.loaders.IAssetLoader;
    import nq.net.assets.storage.IAssetBundle;
    import nq.utils.task.IAsyncTask;

    internal final class LoadAssetTask extends EventDispatcher implements IAsyncTask {

        private var _storage:IAssetBundle;
        private var _loader:IAssetLoader;
        private var _url:String;
        private var _priority:int;
        private var _mimeType:String;

        /**
         * @param storage   Хранилище, в которое нужно положить результат
         * @param loader    Загрузчик ресурса
         * @param url       URL-адрес ресурса для загрузки
         */
        public function LoadAssetTask(storage:IAssetBundle, loader:IAssetLoader, url:String, priority:int, mimeType:String) {

            super();

            if (!storage) throw new ArgumentError("Параметр не должен быть нулевым!");
            if (!loader) throw new ArgumentError("Параметр не должен быть нулевым!");
            if (!url) throw new ArgumentError("Параметр не должен быть нулевым!");

            _storage = storage;
            _loader = loader;
            _url = url;
            _priority = priority;
            _mimeType = mimeType;
        }

        private function onCancel(event:Event):void {

            this.stop();

            super.dispatchEvent(new Event(Event.CANCEL));
        }

        private function onComplete(event:Event):void {

            this.stop();

            _storage.addAsset(_loader.data, _url);

            super.dispatchEvent(new Event(Event.COMPLETE));

        }

        public function get url():String {
            return _url;
        }


        /* INTERFACE nq.utils.task.IAsyncTask */

        /**
		 * Остановить выполнение задания
		 */
        public function stop():void {

            _loader.removeEventListener(Event.COMPLETE, onComplete);
            _loader.removeEventListener(Event.CANCEL, onCancel);

            _loader.close();

        }

        /**
		 * Проритет задания
		 */
        public function get priority():int {
            return _priority;
        }

        public function get mimeType():String
        {
            return _mimeType;
        }

        /**
		 * Начать выполнение задания
		 */
        public function start():void {

            _loader.addEventListener(Event.COMPLETE, onComplete);
            _loader.addEventListener(Event.CANCEL, onCancel);
            _loader.load(_url);

        }

    }

}
